@@ -35,12 +35,16 @@ cond2sum <- function(formula) {
 #'   a vector of values and return either a (possibly named) single value,
 #'   a (possibly named) vector of values, or a data frame with one row.
 #'   Functions can be specified with character strings, names, or expressions
-#'   that look like function calls wit the first argument missing.  The latter
+#'   that look like function calls with the first argument missing.  The latter
 #'   option provides a convenient way to specify additional arguments.  See the
 #'   examples.
 #'   Note: If these arguments are named, those names will be used in the data
 #'   frame returned (see details).  Such names may not be among the names of the named
 #'   arguments of `df_stats`().
+#'
+#'   If a function is specified using `::`, be sure to include the trailing
+#'   parens, even if there are no additional arguments required.
+#'
 #' @param groups An expression to be evaluated in `data` and defining (additional) groups.
 #'   This isn't necessary, since these can be placed into the formula, but it is provided
 #'   for similarity to other functions from the \pkg{mosaic} package.
@@ -81,13 +85,7 @@ cond2sum <- function(formula) {
 #' Note that unlike \code{dplyr::\link[dplyr]{summarise}()}, `df_stats()` ignores
 #' any grouping defined in `data` if `data` is a grouped `tibble`.
 #'
-#' Names of columns in the resulting data frame are determined as follows.  For named
-#' arguments in `...`, the argument name is used.  For unnamed arguments, if the
-#' statistic function returns a result with names, those names are used.  Else, a name is
-#' computed from the expression in `...` and the name of the variable being summarized.
-#' For functions that produce multiple
-#' outputs without names, consecutive integers are appended to the names.
-#' See the examples.
+
 #'
 #' @section Cautions Regarding Formulas:
 #'
@@ -101,13 +99,24 @@ cond2sum <- function(formula) {
 #' Similarly, addition (`+`) is used to separate grouping variables, not for
 #' arithmetic.
 #'
-#' @return A data frame.
+#' @return A data frame. Names of columns in the resulting data frame consist of three
+#'   parts separated by `sep`.
+#'   The first part is the argument name, if it exists, else the function.
+#'   The second part is the name of the variable being summarised if `long_names == TRUE` and
+#'   the first part is the function name, else ""
+#'   The third part is the names of the object returned by the summarizing function, if they
+#'   exist, else a sequence of consecutive integers or "" if there is only one component
+#'   returned by the summarizing function.
+#'   See the examples.
 #'
 #' @examples
 #' df_stats( ~ hp, data = mtcars)
 #' # There are several ways to specify functions
 #' df_stats( ~ hp, data = mtcars, mean, trimmed_mean = mean(trim = 0.1), "median",
 #'   range, Q = quantile(c(0.25, 0.75)))
+#' # When using ::, be sure to include parents, even if there are no additional arguments.
+#' df_stats( ~ hp, data = mtcars, mean = base::mean(), trimmed_mean = base::mean(trim = 0.1))
+#'
 #' # force names to by syntactically valid
 #' df_stats( ~ hp, data = mtcars, Q = quantile(c(0.25, 0.75)), nice_names = TRUE)
 #' # shorter names
@@ -120,15 +129,28 @@ cond2sum <- function(formula) {
 #' df_stats( hp ~ cyl + gear, data = mtcars, mean, median, range)
 #' df_stats( hp ~ cyl | gear, data = mtcars, mean, median, range)
 #' df_stats( hp ~ cyl, groups = gear, data = mtcars, mean, median, range)
-#' # magrittr style piping is also supported
+#'
+#' # because the result is a data frame, df_stats() is also useful for creating plots
 #' if(require(ggformula)) {
-#'   mtcars %>%
-#'   df_stats(hp ~ cyl)
 #'   gf_violin(hp ~ cyl, data = mtcars, group = ~ cyl) %>%
-#'   gf_point(mean_hp ~ cyl, data = df_stats(hp ~ cyl, data = mtcars, mean))
+#'   gf_point(mean_hp ~ cyl, data = df_stats(hp ~ cyl, data = mtcars, mean),
+#'     color = ~ "mean") %>%
+#'   gf_point(median_hp ~ cyl, data = df_stats(hp ~ cyl, data = mtcars, median),
+#'     color = ~"median") %>%
+#'   gf_labs(color = "")
 #' }
 #'
-#' # can be used on a categorical response, too
+#' # magrittr style piping is also supported
+#' if (require(ggformula)) {
+#'   mtcars %>%
+#'     df_stats(hp ~ cyl, mean, median, range)
+#'   mtcars %>%
+#'     df_stats(hp ~ cyl + gear, mean, median, range) %>%
+#'     gf_point(mean_hp ~ cyl, color = ~ factor(gear)) %>%
+#'     gf_line(mean_hp ~ cyl, color = ~ factor(gear))
+#' }
+#'
+#' # can be used with a categorical response, too
 #' if (require(mosaic)) {
 #'   df_stats(sex ~ substance, data = HELPrct, table, prop_female = prop)
 #' }
@@ -144,12 +166,13 @@ df_stats <- function(formula, data, ..., drop = TRUE, fargs = list(),
                      format = c("wide", "long"), groups = NULL,
                      long_names = TRUE, nice_names = FALSE,
                      na.action = "na.warn") {
-  qdots <- quos(...)
+
+  qdots <- dplyr::quos(...)
   # dots <- rlang::exprs(...)
   format <- match.arg(format)
 
   if (length(qdots) < 1) {
-    qdots <- list(rlang::quo(gf_favstats))
+    qdots <- list(dplyr::quo(df_favstats))
     names(qdots) <- ""
     na.action = "na.pass"
   }
@@ -163,7 +186,7 @@ df_stats <- function(formula, data, ..., drop = TRUE, fargs = list(),
   if ( ! inherits(formula, "formula")) stop("first arg must be a formula")
   if ( ! inherits(data, "data.frame")) stop("second arg must be a data.frame")
 
-  formula <- cond2sum(mosaic_formula_q(formula, groups = groups))
+  formula <- cond2sum(mosaic_formula_q(reop_formula(formula), groups = groups))
 
   if (identical(na.action, "na.warn")) na.action <- na.warn
 
@@ -186,11 +209,11 @@ df_stats <- function(formula, data, ..., drop = TRUE, fargs = list(),
       qdots,
       function(f) {
         if (inherits(rlang::f_rhs(f), "call")) {
-          aggregate(MF[, 1], by = MF[, -1, drop = FALSE],
+          df_aggregate(MF[, 1], by = MF[, -1, drop = FALSE],
                     FUN = function(x) eval(substitute(x %>% foo, list(foo = rlang::f_rhs(f)))),
                     drop = drop)
         } else {
-          aggregate(MF[, 1], by = MF[, -1, drop = FALSE],
+          df_aggregate(MF[, 1], by = MF[, -1, drop = FALSE],
                     FUN = function(x) do.call(rlang::eval_tidy(f), c(list(x), fargs)),
                     drop = drop)
         }
@@ -230,27 +253,38 @@ df_stats <- function(formula, data, ..., drop = TRUE, fargs = list(),
       }
       )
 
-  if (long_names) {
-    fun_names <- paste0(fun_names, sep, deparse(formula[[2]]))
-  }
+  # ignore function names if results are named.
+
   fun_names <- ifelse(sapply(res_names, is.null), fun_names, "")
 
-  # # Use numbers or "" if there are no names.
-  alt_res_names <- lapply(ncols, function(nc) if (nc > 1) format(1:nc) else "")
+  # part1: argument name if exists, else function name
+  part1 <-
+    rep(ifelse(arg_names == "", fun_names, arg_names), ncols)
+  part1 <- gsub("df_favstats", "", part1)
 
+  # part2: variable name or ""
+  part2 <-
+    rep(ifelse(arg_names == "" & long_names & ! fun_names == "df_favstats",
+               deparse(formula[[2]]), ""),
+        ncols)
+  # no part2 if part1 is empty
+  part2 <-
+    ifelse(part1 == "", "", part2)
+
+  # part3: res_names if they exist, else numbers or ""
   res_names <-
+    ifelse(sapply(res_names, is.null), "", res_names)
+  alt_res_names <- lapply(ncols, function(nc) if (nc > 1) format(1:nc) else "")
+  part3 <-
     mapply(
-      function(x, y) { if (is.null(x) || x == "") y else x },
+      function(r, a) { if (is.null(r) || r == "") a else r },
       res_names, alt_res_names
     )
+  part3 <- unlist(part3)
 
+  # paste it all together
   final_names <-
-    mapply(
-      paste0,
-      ifelse(arg_names == "", fun_names, arg_names),
-      sep,
-      res_names) %>%
-    unlist()
+    paste(part1, part2, part3, sep = sep)
 
   # remove unneccessary seperators
   final_names <- gsub(paste0(sep, sep), sep, final_names)
@@ -276,7 +310,7 @@ df_stats <- function(formula, data, ..., drop = TRUE, fargs = list(),
   }
 }
 
-gf_favstats <- function (x, ..., na.rm = TRUE, type = 7)
+df_favstats <- function (x, ..., na.rm = TRUE, type = 7)
 {
   if (!is.null(dim(x)) && min(dim(x)) != 1)
     warning("Not respecting matrix dimensions.  Hope that's OK.")
